@@ -128,7 +128,8 @@ Q_SIGNALS:
 public Q_SLOTS:
     void quote(QString id, QString symbol, QDate date, double price)
     {
-        alkDebug() << "comparing quote" << id << symbol;
+        alkDebug() << "comparing quote"
+                   << "with id" << id << "and symbol" << symbol;
         QCOMPARE(date, _date);
         QCOMPARE(price, _price);
         Q_EMIT finished();
@@ -164,7 +165,8 @@ Q_SIGNALS:
 public Q_SLOTS:
     void quotes(const QString& id, const QString& symbol, const AlkDatePriceMap& prices)
     {
-        alkDebug() << "comparing" << prices.size() << "quotes" << id << symbol;
+        alkDebug() << "comparing" << prices.size() << "quotes"
+                   << "with id" << id << "and symbol" << symbol;
         QCOMPARE(prices.size(), _prices.size());
         QCOMPARE(prices.keys(), _prices.keys());
         for (auto &date: prices.keys()) {
@@ -213,30 +215,83 @@ void AlkOnlineQuotePrivateTest::testParseQuoteCSVDateRange()
     QVERIFY(p.parseQuoteCSV(quotedata));
 }
 
-void AlkOnlineQuotePrivateTest::testDateRangeInCSVUrls()
+void AlkOnlineQuotePrivateTest::testDateRangeInUrls()
 {
     AlkOnlineQuote::Private &p = d_ptr();
 
     p.m_startDate = QDate::fromString("21-01-2024", "dd-MM-yyyy");
     p.m_endDate = QDate::fromString("23-01-2024", "dd-MM-yyyy");
 
+    QDateTime startDate = QDateTime(p.m_startDate, QTime(), Qt::LocalTime);
+    QDateTime endDate = QDateTime(p.m_endDate, QTime(23,59, 59, 999), Qt::LocalTime);
+    qint64 startTimeUnix = startDate.toMSecsSinceEpoch() / 1000;
+    qint64 endTimeUnix = endDate.toMSecsSinceEpoch() / 1000;
+
     QUrl url("http://unknown.host/path?start=%unix&end=%unix");
     QVERIFY(p.applyDateRange(url));
     QString urlStr = url.toEncoded();
 
-    QDateTime startDate = QDateTime(p.m_startDate, QTime(), Qt::LocalTime);
-    QDateTime endDate = QDateTime(p.m_endDate, QTime(23,59, 59, 999), Qt::LocalTime);
-    qint64 startUnixTime = startDate.toMSecsSinceEpoch() / 1000;
-    qint64 endUnixTime = endDate.toMSecsSinceEpoch() / 1000;
-
-    QVERIFY(urlStr.contains(QString(QLatin1String("start=%1")).arg(startUnixTime)));
-    QVERIFY(urlStr.contains(QString(QLatin1String("end=%1")).arg(endUnixTime)));
+    QVERIFY(urlStr.contains(QString(QLatin1String("start=%1")).arg(startTimeUnix)));
+    QVERIFY(urlStr.contains(QString(QLatin1String("end=%1")).arg(endTimeUnix)));
 
     url.setUrl("http://unknown.host/path?start=%unix&end=%unix&some=%unix");
     QVERIFY(!p.applyDateRange(url));
 
+    // only start date is defined
     url.setUrl("http://unknown.host/path?start=%unix");
-    QVERIFY(!p.applyDateRange(url));
+    QVERIFY(p.applyDateRange(url));
+
+    url.setUrl("http://unknown.host/path?start=%u&end=%u");
+    QVERIFY(p.applyDateRange(url));
+    urlStr = url.toEncoded();
+
+    QVERIFY(urlStr.contains(QString(QLatin1String("start=%1")).arg(startTimeUnix)));
+    QVERIFY(urlStr.contains(QString(QLatin1String("end=%1")).arg(endTimeUnix)));
+
+    // utc time zone
+    QDateTime startDateUTC = QDateTime(p.m_startDate, QTime(), Qt::UTC);
+    QDateTime endDateUTC = QDateTime(p.m_endDate, QTime(23,59, 59, 999), Qt::UTC);
+    qint64 startTimeUTC = startDateUTC.toMSecsSinceEpoch() / 1000;
+    qint64 endTimeUTC = endDateUTC.toMSecsSinceEpoch() / 1000;
+
+    url.setUrl("http://unknown.host/path?start=%utc&end=%utc");
+    QVERIFY(p.applyDateRange(url));
+    urlStr = url.toEncoded();
+
+    QVERIFY(urlStr.contains(QString(QLatin1String("start=%1")).arg(startTimeUTC)));
+    QVERIFY(urlStr.contains(QString(QLatin1String("end=%1")).arg(endTimeUTC)));
+
+    // end date limit
+    p.m_endDate = QDate::currentDate();
+    endDate = QDateTime(p.m_endDate, QTime(23,59, 59, 999), Qt::LocalTime);
+    endTimeUnix = endDate.toMSecsSinceEpoch() / 1000;
+
+    url.setUrl("http://unknown.host/path?start=%u&end=%u");
+    QVERIFY(p.applyDateRange(url));
+    urlStr = url.toEncoded();
+
+    QVERIFY(!urlStr.contains(QString(QLatin1String("end=%1")).arg(endTimeUnix)));
+}
+
+void AlkOnlineQuotePrivateTest::testParseQuoteJson()
+{
+    AlkOnlineQuote::Private &p = d_ptr();
+
+    AlkOnlineQuoteSource source("test", "", "", AlkOnlineQuoteSource::Symbol, "chart:result:indicators:quote:open", "chart:result:timestamp", "%u", AlkOnlineQuoteSource::JSON);
+    p.m_source = source;
+    p.m_startDate = QDate::fromString("21-10-2024", "dd-MM-yyyy");
+    p.m_endDate = QDate::fromString("25-10-2024", "dd-MM-yyyy");
+
+    QFile f(":/alkonlinequoteprivatetest.json");
+    QVERIFY(f.open(QIODevice::ReadOnly));
+    QString quotedata = f.readAll();
+
+    AlkDatePriceMap map;
+    map[QDate::fromString("23-10-2024", "dd-MM-yyyy")] = 61807.7109375;
+    map[QDate::fromString("24-10-2024", "dd-MM-yyyy")] = 61798.4375;
+    MultipleQuotesReceiver multiReceiver(map);
+    connect(this, SIGNAL(quotes(QString,QString,AlkDatePriceMap)), &multiReceiver, SLOT(quotes(QString,QString,AlkDatePriceMap)));
+    QVERIFY(p.parseQuoteJson(quotedata));
 }
 
 #include "alkonlinequoteprivatetest.moc"
