@@ -52,6 +52,9 @@ using Regex = QRegularExpression;
 
 #ifndef I18N_NOOP
 #include <KLazyLocalizedString>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
 #endif
 
 AlkOnlineQuote::Private::Private(AlkOnlineQuote *parent)
@@ -76,48 +79,73 @@ AlkOnlineQuote::Private::~Private()
         delete m_profile;
 }
 
+QString replaceVariableInUrl(const QString &urlStr, const QStringList &pattern, const QString &replace)
+{
+    QString result = urlStr;
+    for (const QString &p: pattern) {
+        int n = p.length();
+        int i = result.indexOf(p);
+        if (i != -1) {
+            result.replace(i, n, replace);
+            return result;
+        }
+    }
+    return result;
+}
+
+QString replaceStartAndEndDateTime(const QString &urlStr, const QStringList &pattern, const QDateTime &startDate, const QDateTime &endDate)
+{
+    qint64 startUnixTime = startDate.toMSecsSinceEpoch() / 1000;
+    qint64 endUnixTime = endDate.toMSecsSinceEpoch() / 1000;
+
+    alkDebug() << startUnixTime << endUnixTime;
+
+    QString startNumber = QString::number(startUnixTime);
+    QString endNumber = QString::number(endUnixTime);
+
+    alkDebug()
+       << "apply date range from" << QString("%1 (%2)").arg(startDate.toString(Qt::ISODate), startNumber)
+       << "to" << QString("%1 (%2)").arg(endDate.toString(Qt::ISODate), endNumber)
+       << "to pattern" << pattern;
+
+    QString result(urlStr);
+    result = replaceVariableInUrl(result, pattern, startNumber);
+    result = replaceVariableInUrl(result, pattern, endNumber);
+    return result;
+}
+
 bool AlkOnlineQuote::Private::applyDateRange(QUrl &url)
 {
     QString urlStr(url.toEncoded());
-    int i = -1;
-    int j = -1;
+    QStringList patternUTC = { "%25utc", "%utc" };
+    QStringList pattern = { "%25unix", "%unix", "%25u", "%u" };
+
     if (m_startDate.isValid() && m_endDate.isValid()) {
-        QDateTime startDate = QDateTime(m_startDate, QTime(), Qt::LocalTime);
-        QDateTime endDate = QDateTime(m_endDate, QTime(23,59,59, 999), Qt::LocalTime);
-        qint64 startUnixTime = startDate.toMSecsSinceEpoch() / 1000;
-        qint64 endUnixTime = endDate.toMSecsSinceEpoch() / 1000;
+        QDateTime startDateLocal = QDateTime(m_startDate, QTime(), Qt::LocalTime);
+        QDateTime endDateLocal = QDateTime(m_endDate, QTime(23,59,59, 999), Qt::LocalTime);
 
-        alkDebug() << startUnixTime << endUnixTime;
 
-        QString startNumber = QString::number(startUnixTime);
-        QString endNumber = QString::number(endUnixTime);
+        QDateTime startDateUTC = QDateTime(m_startDate, QTime(), Qt::UTC);
+        QDateTime endDateUTC = QDateTime(m_endDate, QTime(23,59,59, 999), Qt::UTC);
 
-        alkDebug() << startNumber << endNumber;
+        // do not ask for times in the future
+        if (endDateLocal > QDateTime::currentDateTime())
+                endDateLocal = QDateTime::currentDateTime();
 
-        i = urlStr.indexOf(QLatin1String("%25unix"));
-        if (i != -1)
-            urlStr.replace(i, 7, startNumber);
-        else {
-            i = urlStr.indexOf(QLatin1String("%unix"));
-            if (i != -1)
-                urlStr.replace(i, 5, startNumber);
-        }
+        if (endDateUTC > QDateTime::currentDateTimeUtc())
+                endDateUTC = QDateTime::currentDateTimeUtc();
 
-        j = urlStr.indexOf(QLatin1String("%25unix"));
-        if (j != -1)
-            urlStr.replace(j, 7, endNumber);
-        else {
-            j = urlStr.indexOf(QLatin1String("%unix"));
-            if (j != -1)
-                urlStr.replace(j, 5, endNumber);
-        }
+        urlStr = replaceStartAndEndDateTime(urlStr, patternUTC, startDateUTC, endDateUTC);
+        urlStr = replaceStartAndEndDateTime(urlStr, pattern, startDateLocal, endDateLocal);
+        pattern << patternUTC;
     }
 
     alkDebug() << urlStr;
 
-    // return error if only one or more than two placeholders are present
-    if (urlStr.contains(QLatin1String("%25unix")) || urlStr.contains(QLatin1String("%unix")) || (i != -1 && j == -1)) {
-        return false;
+    // Returns an error if one of the searched patterns is still present in the url
+    for (QString &p: pattern) {
+        if (urlStr.contains(p))
+            return false;
     }
 
     url = QUrl::fromEncoded(urlStr.toLocal8Bit());
@@ -165,6 +193,8 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
         // if we've truly found 2 symbols delimited this way...
         if (splitrx.indexIn(m_symbol) != -1) {
             url = KUrl(m_source.url().arg(splitrx.cap(1), splitrx.cap(2)));
+            m_symbols[0] = splitrx.cap(1);
+            m_symbols[1] = splitrx.cap(2);
         } else {
             alkDebug() << QString("AlkOnlineQuote::Private::initLaunch() did not find 2 symbols in '%1'").arg(m_symbol);
         }
@@ -174,6 +204,8 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
         // if we've truly found 2 symbols delimited this way...
         if (match.hasMatch()) {
             url = KUrl(m_source.url().arg(match.captured(1), match.captured(2)));
+            m_symbols = match.capturedTexts();
+            m_symbols.takeFirst();
         } else {
             alkDebug() << QString("AlkOnlineQuote::Private::initLaunch() did not find 2 symbols in '%1'").arg(m_symbol);
         }
@@ -196,6 +228,40 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
     return true;
 }
 
+bool AlkOnlineQuote::Private::launch(const QString &symbol, const QString &id, const QString &source)
+{
+    bool result = false;
+#ifdef ENABLE_FINANCEQUOTE
+    if (AlkOnlineQuoteSource::isFinanceQuote(source) ||
+            m_profile->type() == AlkOnlineQuotesProfile::Type::Script) {
+        result = launchFinanceQuote(symbol, id, source);
+    } else
+#endif
+    if (!initSource(source))
+        return false;
+
+    if (m_source.downloadType() == AlkOnlineQuoteSource::Javascript) {
+        result = launchWithJavaScriptSupport(symbol, id, AlkDownloadEngine::JavaScriptEngine);
+    } else if (m_source.dataFormat() == AlkOnlineQuoteSource::CSS) {
+        result = launchWithJavaScriptSupport(symbol, id, AlkDownloadEngine::JavaScriptEngineCSS);
+    } else {
+        result = launchNative(symbol, id);
+    }
+    if (!result && m_source.requiresTwoIdentifier() && m_reverseLaunchEnabled) {
+        m_reverseLaunch = true;
+        QString newSymbol(QString("%1 %2").arg(m_symbols[1], m_symbols[0]));
+        if (m_source.downloadType() == AlkOnlineQuoteSource::Javascript) {
+            result = launchWithJavaScriptSupport(newSymbol, id, AlkDownloadEngine::JavaScriptEngine);
+        } else if (m_source.dataFormat() == AlkOnlineQuoteSource::CSS) {
+            result = launchWithJavaScriptSupport(newSymbol, id, AlkDownloadEngine::JavaScriptEngineCSS);
+        } else {
+            result = launchNative(newSymbol, id);
+        }
+        m_reverseLaunch = false;
+    }
+    return result;
+}
+
 void AlkOnlineQuote::Private::slotLoadError(const QUrl &, const QString &)
 {
     Q_EMIT m_p->error(i18n("Unable to fetch url for %1", m_symbol));
@@ -407,6 +473,8 @@ bool AlkOnlineQuote::Private::parsePrice(const QString &_pricestr, AlkOnlineQuot
         bool ok;
         m_price = pricestr.toDouble(&ok);
         if (ok) {
+            if (m_reverseLaunch)
+                m_price = 1.0 / m_price;
             alkDebug() << "Price" << pricestr;
             Q_EMIT m_p->status(i18n("Price found: '%1' (%2)", pricestr, m_price));
         } else {
@@ -683,6 +751,11 @@ bool AlkOnlineQuote::Private::parseQuoteCSV(const QString &quotedata)
             priceValue.replace(QLatin1Char(','), QLatin1Char('.'));
 
         AlkValue price = AlkValue(priceValue, decimalSeparator);
+        if (price == AlkValue(0))
+            continue;
+        if (m_reverseLaunch)
+            price = AlkValue(1.0) / price;
+
         if (lastDate.isNull() || lastDate <= date) {
             lastPrice = price;
             lastDate = date;
@@ -720,6 +793,148 @@ bool AlkOnlineQuote::Private::parseQuoteCSV(const QString &quotedata)
     return true;
 }
 
+/**
+ * Parse quote data in json format
+ *
+ * @param quotedata quote data to parse
+ * @return true parsing successful
+ * @return false parsing unsuccessful
+ */
+bool AlkOnlineQuote::Private::parseQuoteJson(const QString &quotedata)
+{
+    QString dateHierachy(m_source.dateRegex());
+    QString priceHierachy(m_source.priceRegex());
+    auto jsonDoc = QJsonDocument::fromJson(quotedata.toLocal8Bit());
+
+    if (jsonDoc.isNull()) {
+        m_errors |= Errors::Source;
+        alkDebug() << "Failed to create JSON doc";
+    } else if (!jsonDoc.isObject()) {
+        m_errors |= Errors::Source;
+        alkDebug() << "JSON is not an object";
+    }
+
+    QJsonObject json = jsonDoc.object();
+
+    if (json.isEmpty()) {
+        m_errors |= Errors::Source;
+        alkDebug() << "JSON object is empty.";
+    }
+
+    if (!m_errors.isEmpty() && m_errors != Errors::None && m_errors != Errors::Success) {
+        Q_EMIT m_p->error(i18n("Json file with invalid content found for '%1'", m_symbol));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+
+    QJsonObject o = json;
+    QJsonObject o1;
+    QJsonArray a;
+    QString s;
+
+    // extract dates
+    QList<int> dateList;
+    QStringList keyList = dateHierachy.split(":");
+    QString key = keyList.takeFirst();
+    while (!o.isEmpty()) {
+        if (o.contains(key) && o[key].isObject()) {
+            o = o[key].toObject();
+            key = keyList.takeFirst();
+        } else if (o.contains(key) && o[key].isArray()) {
+            a = o[key].toArray();
+            // requested level has been reached
+            if (keyList.size() == 0) {
+                for (const auto &b : qAsConst(a)) {
+                    if (b.toInt())
+                        dateList.append(b.toInt());
+                }
+                break;
+            } else {
+                key = keyList.takeFirst();
+                for (const auto &b : qAsConst(a)) {
+                    if (b.isObject())
+                        o = b.toObject();
+                }
+            }
+        } else if (o.contains(key) && o[key].isString())
+            s = o[key].toString();
+    }
+
+    // extract prices
+    o = json;
+    QList<double> priceList;
+    keyList = priceHierachy.split(":");
+    key = keyList.takeFirst();
+    while (!o.isEmpty()) {
+        if (o.contains(key) && o[key].isObject()) {
+            o = o[key].toObject();
+            key = keyList.takeFirst();
+        } else if (o.contains(key) && o[key].isArray()) {
+            a = o[key].toArray();
+            // requested level has been reached
+            if (keyList.size() == 0) {
+                for (const auto &b : qAsConst(a)) {
+                    if (b.toDouble())
+                        priceList.append(b.toDouble());
+                    else if (b.isNull())
+                        priceList.append(0.0);
+                }
+                break;
+            } else {
+                key = keyList.takeFirst();
+                for (const auto &b : qAsConst(a)) {
+                    if (b.isObject())
+                        o = b.toObject();
+                }
+            }
+        } else if (o.contains(key) && o[key].isString())
+            s = o[key].toString();
+    }
+
+    AlkDatePriceMap prices;
+    AlkDateFormat dateFormat(m_source.dateFormat());
+    int count = std::min(dateList.size(), priceList.size());
+    for (int i = 0; i < count; i++) {
+        int dateValue = dateList.at(i);
+        double priceValue = priceList.at(i);
+        if (priceValue == 0.0)
+            continue;
+        QDate date = dateFormat.convertString(QString("%1").arg(dateValue), false);
+        if (!date.isValid()) {
+            m_errors |= Errors::DateFormat;
+            Q_EMIT m_p->error(i18n("Unable to convert date '%1' with '%2' in quote data", dateValue, m_source.dateFormat()));
+            Q_EMIT m_p->failed(m_id, m_symbol);
+            return false;
+        }
+        if (!m_startDate.isNull() && date < m_startDate)
+            continue;
+        if (!m_endDate.isNull() && date > m_endDate)
+            continue;
+
+        AlkValue price = AlkValue(priceValue);
+        if (m_reverseLaunch)
+            price = AlkValue(1.0) / price;
+        prices[date] = price;
+    }
+
+    if (prices.isEmpty()) {
+        m_errors |= Errors::Price;
+        Q_EMIT m_p->error(i18n("Unable to find date/price pairs in quote data"));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+
+    if (m_useSingleQuoteSignal) {
+        for (auto &key : prices.keys()) {
+            Q_EMIT m_p->quote(m_id, m_symbol, key, prices[key].toDouble());
+        }
+    } else {
+        Q_EMIT m_p->quotes(m_id, m_symbol, prices);
+    }
+
+    return true;
+}
+
 /**
  * Parse quote data according to currently selected web price quote source
  *
@@ -746,6 +961,8 @@ bool AlkOnlineQuote::Private::slotParseQuote(const QString &quotedata)
         return parseQuoteHTML(quotedata);
     case AlkOnlineQuoteSource::CSV:
         return parseQuoteCSV(quotedata);
+    case AlkOnlineQuoteSource::JSON:
+        return parseQuoteJson(quotedata);
     default:
         return false;
     }
